/*

k8055d.cpp

by Matthew Williams

$Revision: 23 $
$Date: 2006-01-20 14:11:22 -0500 (Fri, 20 Jan 2006) $

*/

#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <signal.h>
#include <pthread.h>
#include <usb.h>
#include <iostream>
#include <errno.h>
#include <sys/mman.h>

#include "ueib.h"

using namespace std;

#define MAX_MESSAGE_LENGTH 2048
#define MESSAGE_BUFFER_SIZE (MAX_MESSAGE_LENGTH+1)
#define MAX_WORDS 10

#define WITHIN(a,b,c) (((a) >= (b)) && ((a) <= (c)))

bool inBackground=false;
pthread_mutex_t mutex;
ueib_handle *uh = 0;

#ifdef FAKE_INTERFACE
char analogueValue=1;
#endif

void lockMutex(void) {
	pthread_mutex_lock(&mutex);
}

void unlockMutex(void) {
	pthread_mutex_unlock(&mutex);
}

void updateValues(void) {
	lockMutex();
	#ifndef FAKE_INTERFACE
	ueib_update(uh);
	#endif
	unlockMutex();
}

int readAnalogue(int portNum) {
	int value=0;
	lockMutex();
	#ifdef FAKE_INTERFACE
	value=analogueValue++;
	if (analogueValue > 20) {
		analogueValue=0;
	}
	#else
	value=ueib_get_ana_input(uh,portNum==1?UEIB_ANA1:UEIB_ANA2);
	#endif
	unlockMutex();
	return (value);
	}

void writeAnalogue(int portNum, int value) {
	lockMutex();
	#ifndef FAKE_INTERFACE
	ueib_set_ana_output(uh,portNum==1?UEIB_ANA1:UEIB_ANA2,value);
	#endif
	unlockMutex();
}

void writeDigital(int portNum, int value) {
	int ueibPortNum=0;
	switch(portNum) {
		case 1:
			ueibPortNum=UEIB_OUT1;
			break;
		case 2:
			ueibPortNum=UEIB_OUT2;
			break;
		case 3:
			ueibPortNum=UEIB_OUT3;
			break;
		case 4:
			ueibPortNum=UEIB_OUT4;
			break;
		case 5:
			ueibPortNum=UEIB_OUT5;
			break;
		case 6:
			ueibPortNum=UEIB_OUT6;
			break;
		case 7:
			ueibPortNum=UEIB_OUT7;
			break;
		case 8:
			ueibPortNum=UEIB_OUT8;
			break;
		default:
			return;
			break;
	}
	lockMutex();
	#ifndef FAKE_INTERFACE
	ueib_set_dig_output(uh, ueibPortNum, value);
	#endif
	unlockMutex();
}

int readCounter (int portNum) {
	int value=0;
	lockMutex();
	#ifdef FAKE_INTERFACE
	value=portNum;
	#else
	value=ueib_get_count_input(uh,portNum==1?UEIB_COUNT1:UEIB_COUNT2);
	#endif
	unlockMutex();
	return value;
}

void resetCounter (int portNum) {
	lockMutex();
	#ifndef FAKE_INTERFACE
	ueib_reset_count_input(uh,portNum==1?UEIB_COUNT1:UEIB_COUNT2);
	#endif
	unlockMutex();
}

void setDebounce (int portNum, int debounceTime) {
	lockMutex();
	#ifndef FAKE_INTERFACE
	ueib_set_debounce(uh, portNum==1?UEIB_COUNT1:UEIB_COUNT2, debounceTime);
	#endif
	unlockMutex();
}

int readDigital(int portNum) {
	int value=0;
	int ueibPortNum=0;
	switch(portNum) {
		case 1:
			ueibPortNum=UEIB_INP1;
			break;
		case 2:
			ueibPortNum=UEIB_INP2;
			break;
		case 3:
			ueibPortNum=UEIB_INP3;
			break;
		case 4:
			ueibPortNum=UEIB_INP4;
			break;
		case 5:
			ueibPortNum=UEIB_INP5;
			break;
		default:
			return 0;
			break;
	}

	lockMutex();
	#ifdef FAKE_INTERFACE
	value=20*portNum;
	#else
	value=ueib_get_dig_input(uh,ueibPortNum);
	#endif
	unlockMutex();
	return value;

}

int processCommand(char *command, char *response, int maxLength) {
	int maxWord=0;
	char *word[MAX_WORDS];
	bool inWord=false;
	int timeToDie=0;

	response[0]=0;

	maxLength--; // to account for \n at end of response

	snprintf(response,maxLength-1,"error unknown/invalid command \"%s\"",command);

	unsigned int commandLength=strnlen(command,maxLength);
	for (unsigned int i=0; i < commandLength; i++) {
		if (command[i] == ' ') {
			inWord=false;
			command[i] = 0;
			continue;
		}
		if ((command[i] == '\n') || (command[i]=='\r')) {
			command[i] = 0;
			break;
		}
		if (inWord) {
			continue;
		}
		if (maxWord == MAX_WORDS) {
			break;
		}
		word[maxWord++]=&command[i];
		inWord=true;
	}

	if (strncmp(word[0],"kill",maxLength)==0) {
		snprintf (response, maxLength-1, "ok confirmation killing server and all clients");
		timeToDie=2;
	} else if (strncmp(word[0],"exit",maxLength)==0) {
		snprintf (response, maxLength-1, "ok confirmation exitting");
		timeToDie=1;
	} else if (strncmp(word[0],"pid",maxLength)==0) {
		snprintf (response, maxLength-1, "ok value server pid %i",getppid());
	} else if (strncmp(word[0],"update",maxLength)==0) {
		updateValues();
		snprintf (response, maxLength-1, "ok confirmation updating readings");
	} else if (strncmp(word[0],"debounce",maxLength)==0) {
		if (maxWord >= 2) {
			int portNum=atoi(word[1]);
			if (WITHIN(portNum,1,2)) {
				int value=atoi(word[2]);
				setDebounce(portNum,value);
				snprintf (response, maxLength-1, "ok confirmation debounce %i set to %i",portNum, value);
			}
		}
	} else if (strncmp(word[0],"reset",maxLength)==0) {
		if (maxWord >= 1) {
			int portNum=atoi(word[1]);
			if (WITHIN(portNum,1,2)) {
				resetCounter(portNum);
				snprintf (response, maxLength-1, "ok confirmation reset counter %i",portNum);
			}
		}
	} else if (strncmp(word[0],"write",maxLength)==0) {
		if (maxWord >= 4) {
			int portNum=atoi(word[2]);
			int value=atoi(word[3]);
			if (strncmp(word[1],"digital",maxLength)==0) {
				if (WITHIN(portNum,1,8)) {
					if (WITHIN(value,0,1)) {
						writeDigital(portNum,value);
						snprintf (response, maxLength-1, "ok confirmation setting digital port %i to %i",portNum,value);
					}
				}
			} else if (strncmp(word[1],"analogue",maxLength)==0) {
				if (WITHIN(portNum,1,2)) {
					if (WITHIN(value,0,255)) {
						writeAnalogue(portNum,value);
						snprintf (response, maxLength-1, "ok confirmation setting analogue port %i to %i",portNum,value);
					}
				}
			}
		}
	} else if (strncmp(word[0],"read",maxLength)==0) {
		if (maxWord >= 2) {
			int portNum=atoi(word[2]);
			if (strncmp(word[1],"digital",maxLength)==0) {
				if (WITHIN(portNum,1,5)) {
					snprintf (response, maxLength-1, "ok value digital %i %i",portNum,readDigital(portNum));
				}

			} else if (strncmp(word[1],"analogue",maxLength)==0) {
				if (WITHIN(portNum,1,2)) {
					snprintf(response,maxLength-1,"ok value analogue %i %i", portNum, readAnalogue(portNum));
				}
			} else if (strncmp(word[1],"counter",maxLength)==0) {
				if (WITHIN(portNum,1,2)) {
					snprintf(response,maxLength-1,"ok value counter %i %i", portNum, readCounter(portNum));
				}
			}
		}
	}
	strncat(response,"\n",1);
	return (timeToDie);
}

void handleConnection (int connectSocket) {
	int receivedBytes=0;
	char portionBuffer[MESSAGE_BUFFER_SIZE];
	char messageBuffer[MESSAGE_BUFFER_SIZE];
	char responseBuffer[MESSAGE_BUFFER_SIZE];
	int messageOffset=0;

	messageBuffer[0]=0;

	while ((receivedBytes=recv(connectSocket, portionBuffer, MAX_MESSAGE_LENGTH, 0)) > 0) {
		portionBuffer[receivedBytes]=0;
		strncat(messageBuffer + messageOffset, portionBuffer, MAX_MESSAGE_LENGTH-messageOffset);
		bool foundCR=false;
		for (int i=0; i < receivedBytes+receivedBytes; i++) {
			if (portionBuffer[i]=='\n') {
				foundCR=true;
				break;
			}
		}
		if (!foundCR) {
			messageOffset+=receivedBytes;
		} else {
			char singleMessage[MESSAGE_BUFFER_SIZE];
			int bytesInBuffer=messageOffset+receivedBytes;
			int startOfSingleMessage=0;

			for (int i=0; i < bytesInBuffer; i++) {
				singleMessage[i-startOfSingleMessage]=messageBuffer[i];
				if (singleMessage[i-startOfSingleMessage]=='\n') {
					singleMessage[i-startOfSingleMessage]=0;
					startOfSingleMessage=i+1; // get ready for next message
					if (messageBuffer[i]=='\r') {
						i++;
						startOfSingleMessage++;
					}


					int timeToDie=processCommand(singleMessage, responseBuffer, MAX_MESSAGE_LENGTH);
					send(connectSocket, responseBuffer, strlen(responseBuffer)+1,0);
					if (timeToDie > 0) {
						if (timeToDie == 1) {
							shutdown (connectSocket,SHUT_RDWR);
							return;
						}
						if (timeToDie == 2) { // everyone dies!
							#ifndef FAKE_INTERFACE
							ueib_close(uh); // done with the USB stuff
							#endif
							kill(0,SIGTERM);
						}
					}
				}
			}
			messageBuffer[0]=0;
			messageOffset=0;
		}
	}
}

void handleError(const char *message, int returnCode=1) {
	if (inBackground==false) {
		cerr << message << endl;
		if (errno > 0) {
			cerr << strerror(errno) << endl;
		}
	}
	exit (returnCode);
}

// parameters are <device number 0..3> <tcp port to listen on>

int main (int argc, char *argv[]) {
	int listenPort=9999;
	int listenSocket=0;
	int connectSocket=0;
	struct sockaddr_in clientAddress;
	socklen_t clientAddressLength=0;
	struct sockaddr_in serverAddress;;
	pid_t childPID;
	int device=0;


	if (argc > 1) { // at least 1 parameter
		if (strncmp(argv[1],"help",4)==0) {
			cout << "k8055d <device number 0..3> <tcp port to listen on>" << endl;
			exit (0);
		}
		device=atoi(argv[1]);
		if (!WITHIN(device,0,3)) {
			handleError("device must be 0..3, use 'help' for help");
		}
	}

	if (argc > 2) { // at least 2 parameters
		listenPort=atoi(argv[2]);
	}

	#ifndef FAKE_INTERFACE

	uh=(ueib_handle *)mmap(0,sizeof(ueib_handle), PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);

	if (uh == 0) {
		handleError("Can't create shared memory segment");
	}

	uh = ueib_init(device, uh);

	if (uh==NULL) {
		handleError("Can't connect to K8055");
	}

	ueib_update(uh);

	#endif

	listenSocket=socket(AF_INET, SOCK_STREAM, 0);

	if (listenSocket < 0) {
		handleError("Can't create listen socket");
	}

	int socketOption=1; // 1=yes
	setsockopt(listenSocket, SOL_SOCKET, SO_REUSEADDR, &socketOption, sizeof(socketOption));

	serverAddress.sin_family = AF_INET;
	serverAddress.sin_addr.s_addr = htonl(INADDR_ANY);
	serverAddress.sin_port=htons(listenPort);

	if (bind(listenSocket,
		(struct sockaddr *) &serverAddress, 
		sizeof(serverAddress)) < 0) {
		handleError("Can't bind listen socket");
	}

	pthread_mutex_init(&mutex,0);

	listen (listenSocket, 5);
	clientAddressLength=sizeof(clientAddress);
	childPID=fork();

	if (childPID == -1) {
		handleError("Can't daemonize myself");
	}
	if (childPID != 0) { // I'm the parent
		cout << "Successfully spawned daemon process for device " << device << " on port " << listenPort << " (PID " << childPID << ")" << endl;
		return 0;  // we're done!
	}
	inBackground=true;
	signal(SIGCLD, SIG_IGN); // allow children to die immediately when terminated

	while (1) {
		connectSocket=accept(listenSocket,
			(struct sockaddr *) &clientAddress,
			&clientAddressLength);

		if (connectSocket < 0) {
			handleError("Can't accept connection");
		}

		//cout << "Connection accepted from " << inet_ntoa(clientAddress.sin_addr);
		//cout << ":" << ntohs (clientAddress.sin_port) << endl;

		childPID=fork();
		if (childPID == -1) {
		  handleError("Can't fork child to handle connection");
		}
		if (childPID == 0) { // I'm the child process
			handleConnection(connectSocket);
			return 0;
		} else { // I'm the parent process
			// do nothing special right now
		}
	}
	return 0;
}
