
# Modified for DalSemi 2438 humidity sensor by Ernie Oporto 8/2002
# From Brian Paulson on 2/2002

# Port is the port that your weather station is connected to.  If your
# weather station is connected to the rest of your 1-wire net, you don't
# need to specify a port because mh will use that by default
# The CHIPS are a listing of all of the chips that make up the weather
# station.  You can get this list by looking at the ini.txt file that
# is generated by the Dallas Semiconductor Weather Station software
# I believe that the first 01 chip should be north and then the rest are
# listed in clockwise order
# By the way, I currently have the Weather Station sitting on my floor
# in the home office because I'm waiting for springtime to mount it outside
# As such, I haven't had a chance to verify that the wind direction and
# wind speed are accurate.

use RRDs;

$weather = new iButton::Weather( CHIPS => [ qw( 0100000364767D64 01000003647679B8 010000036476788F
                                                01000003647677AB 010000036476769C 0100000364767AE1
                                                0100000364767BD6 0100000364767C53 1D00000000C6B14D
                                                120000000AC22A42 1000000025CBA26A) ] );

my $humsensor = new iButton(qw(2600000006CF9FB0));


my $plotdir=$config_parms{iButton_plot};

if ($New_Minute){
    #print_log "TIME: $Time";
    my $temp = $weather->read_temp;
    my $DS2438_tempF;
    my $DS2438_tempC;
    my $DS2438_Vad;
    my $DS2438_Vdd;
    my $RH;
    my $dewpointF;
    my $dewpointC;
    my $windspeed = $weather->read_windspeed;
    my $dir = $weather->read_dir;


    # 
    $DS2438_tempC = $humsensor->Get_Temp_2438; #read DS2438 temp in Celsius
    $DS2438_Vad = $humsensor->Get_Vad_2438;   #read DS2438 Vad
    $DS2438_Vdd = $humsensor->Get_Vdd_2438; #read DS2438 Vdd
    # $DS2438_Vsens = $humsensor->Get_Vsens_2438; #read DS2438 Vsens - not useful here, but other models use it
    #   David Bray says...
    # Here is a snipit of C code that computes dew point:
    # I don't know which units you want so here is both -- outTemp is in F
    #       // compute the dewpoint from relative humidity
    #    outTempC = (outTempF - 32)/9*5;
    #       // evp is the environmental vapor pressure
    #    evp = (humid/100) * 0.611 * exp(17.27 * outTempC/(outTempC + 237.3));
    #       // yes there are 2 different numbers 237.3 and 273.15
    #    dewpointC = (116.9 + 237.3 * log(evp))/(16.78 - log(evp));
    #    dewpointF = dewpointC * 9/5+32;

    # Relative Humidity
    if ((defined $DS2438_Vad) && (defined $DS2438_Vdd))
    {
        $RH = ((($DS2438_Vad/$DS2438_Vdd) - 0.16) / 0.0062) * (1.0546 - 0.00216 * $DS2438_tempC);
        # Saturation Vapor over water
        my $evp = ($RH/100) * 0.611 * exp(17.27 * $DS2438_tempC/($DS2438_tempC + 237.3));
        # temperature to which air must be cooled in order to have condensation
        my $dewpointC = (116.9 + 237.3 * log($evp))/(16.78 - log($evp));

        #Americanize the values
        $dewpointF = ($dewpointC * 9)/5+32;
        $DS2438_tempF = ($DS2438_tempC * 9)/5+32;
        #print_log "DS2438: Vad=". $DS2438_Vad . " Vdd=" . $DS2438_Vdd . " Temp2=" . $DS2438_tempF . " RH=" . sprintf ("%3.2f",$RH); # for debugging
    }
    else
    {
        print_log "DS2438: Vad=". $DS2438_Vad . " Vdd=" . $DS2438_Vdd . " Null Vals Temp2=" . $DS2438_tempF . " Temp=" . $temp;
    }


    if ($Minute % 5)
    {
        #
        RRDs::update("$plotdir/temp.rrd","N:$temp","N:$DS2438_tempF","N:$dewpointF") if (defined $temp && defined $DS2438_tempF && defined $dewpointF);
        RRDs::update("$plotdir/humid.rrd","N:$RH") if defined $RH;
    }
    if (($Minute == 50) && ($Second == 0)) {

        if (defined $temp)
        {
            speak "$temp degrees" if (($Hour > 7) && ($Hour < 21));
            print_log "Temp = $temp F";
        }
        print_log "Temp2    = " . sprintf ("%3.2f",$DS2438_tempF) . " F  Humidity = " . sprintf ("%3.2f",$RH) . " %  Dewpoint = " . sprintf ("%3.2f",$dewpointF) . " F  Speed = $windspeed MPH  Direction = $dir" if defined $dir;
        my $timehr=$Time-10000;
        my $timeday=$Time-86400;
        my $timewk=$Time-604800;

        #Plot temp,temp2,dewpoint
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/temp.gif", "--title=Temperature", "--vertical-label=degrees F", "DEF:mytemp=$plotdir/temp.rrd:temp:AVERAGE","DEF:mytemp2=$plotdir/temp.rrd:temp2:AVERAGE","DEF:mytemp3=$plotdir/temp.rrd:temp3:AVERAGE", "LINE2:mytemp#ff0000:weather station","LINE2:mytemp2#0000ff:humsensor","LINE2:mytemp3#00ff00:dewpoint", "--start=1025144197");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/temphr.gif", "--title=Temperature", "--vertical-label=degrees F", "DEF:mytemp=$plotdir/temp.rrd:temp:AVERAGE","DEF:mytemp2=$plotdir/temp.rrd:temp2:AVERAGE","DEF:mytemp3=$plotdir/temp.rrd:temp3:AVERAGE", "LINE2:mytemp#ff0000:weather station","LINE2:mytemp2#0000ff:humsensor","LINE2:mytemp3#00ff00:dewpoint", "--start=$timehr");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/tempday.gif", "--title=Temperature", "--vertical-label=degrees F", "DEF:mytemp=$plotdir/temp.rrd:temp:AVERAGE","DEF:mytemp2=$plotdir/temp.rrd:temp2:AVERAGE","DEF:mytemp3=$plotdir/temp.rrd:temp3:AVERAGE", "LINE2:mytemp#ff0000:weather station","LINE2:mytemp2#0000ff:humsensor","LINE2:mytemp3#00ff00:dewpoint", "--start=$timeday");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/tempwk.gif", "--title=Temperature", "--vertical-label=degrees F", "DEF:mytemp=$plotdir/temp.rrd:temp:AVERAGE","DEF:mytemp2=$plotdir/temp.rrd:temp2:AVERAGE","DEF:mytemp3=$plotdir/temp.rrd:temp3:AVERAGE", "LINE2:mytemp#ff0000:weather station","LINE2:mytemp2#0000ff:humsensor","LINE2:mytemp3#00ff00:dewpoint", "--start=$timewk");
        #Plot humidity
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/humid.gif", "--title=Humidity", "--vertical-label=%", "DEF:myhumid=$plotdir/humid.rrd:humid:AVERAGE", "LINE2:myhumid#ff0000:humidity", "--start=1025144197");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/humidhr.gif", "--title=Humidity", "--vertical-label=%", "DEF:myhumid=$plotdir/humid.rrd:humid:AVERAGE", "LINE2:myhumid#ff0000:humidity", "--start=$timehr");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/humidday.gif", "--title=Humidity", "--vertical-label=%", "DEF:myhumid=$plotdir/humid.rrd:humid:AVERAGE", "LINE2:myhumid#ff0000:humidity", "--start=$timeday");
        (my $junk1, my $junk2, my $junk3)=RRDs::graph("$plotdir/humidwk.gif", "--title=Humidity", "--vertical-label=%", "DEF:myhumid=$plotdir/humid.rrd:humid:AVERAGE", "LINE2:myhumid#ff0000:humidity", "--start=$timewk");
    }
}

#TODO:
#Heat Index:
#
#' Declare the variables for the heat index.
#    Dim Th
#    Dim RH
#    Dim HI
#' Set the values for the heat index.
#    Th = txtTemperatureHeat.Text
#    RH = txtHumidityHeat.Text
#' Calculate the heat index.
#    HI = 16.923 + (1.85212 * 10 ^ -1 * Th) + (5.37941 * RH) - (1.00254 * 10
#^ -1 * Th * RH) _
#        + (9.41695 * 10 ^ -3 * Th ^ 2) + (7.28898 * 10 ^ -3 * RH ^ 2) +
#(3.45372 * 10 ^ -4 * Th ^ 2 * RH) _
#        - (8.14971 * 10 ^ -4 * Th * RH ^ 2) + (1.02102 * 10 ^ -5 * Th ^ 2 *
#RH ^ 2) - (3.8646 * 10 ^ -5 * Th ^ 3) _
#        + (2.91583 * 10 ^ -5 * RH ^ 3) + (1.42721 * 10 ^ -6 * Th ^ 3 * RH) +
#(1.97483 * 10 ^ -7 * Th * RH ^ 3) _
#        - (2.18429 * 10 ^ -8 * Th ^ 3 * RH ^ 2) + (8.43296 * 10 ^ -10 * Th ^
#2 * RH ^ 3) - (4.81975 * 10 ^ -11 * Th ^ 3 * RH ^ 3)
#' Display the heat index.
#    txtIndexHeat.Text = Format(HI, "##0")
#
#Wind Chill:
#
#' Declare the variables for the wind chill index.
#    Dim Tw
#    Dim V
#    Dim WC
#' Set the values for the wind chill index.
#    Tw = txtTemperatureWind.Text
#    V = txtWindWind.Text
#' Calculate the wind chill index.
#    WC = 91.4 - (0.474677 - 0.020425 * V + 0.303107 * Sqr(V)) * (91.4 - Tw)
#' Display the wind chill index.
#    txtIndexWind.Text = Format(WC, "##0")
#
#It is my understanding that Heat Index values require 80 degrees (F) or
#greater and RH of 40% or greater and Wind Chill values are only valid at
#temperatures below 45 degrees (F) with a wind of 5 MPH or greater.
#
#
